﻿using System;
using System.Collections;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Web.UI.HtmlControls;

namespace DropDownListEx
{

    [ToolboxData("<{0}:DropDownListEx runat=server></{0}:DropDownListEx>")]
    public partial class DropDownListEx : DropDownList, IPostBackDataHandler
    {
        private const int DefaultHeight = 300;

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            if (Page == null)
                throw new InvalidOperationException("We need a page to operate properly");

            if (!Page.IsPostBack)
                SelectedIndex = -1;
        }

        #region Rendering


        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            string script = @"<script language='javascript'> 
	        var dd = new listBoxInstance('" + this.ClientID + @"', " + this.AutoPostBack.ToString().ToLower() + @");
	        dd.doInit();</script>
            ";

            Page.ClientScript.RegisterStartupScript(this.GetType(), "registerDropDownHandlers" + this.ClientID, script);
        }

        public override void RenderBeginTag(HtmlTextWriter writer)
        {
            writer.WriteBeginTag("div");
            writer.WriteAttribute("ID", ClientID);
            writer.WriteAttribute("Name", this.UniqueID);
            if (!string.IsNullOrEmpty(CssClass))
            {
                writer.WriteAttribute("class", "controlWrapper " + CssClass);
            }
            else
            {
                writer.WriteAttribute("class", "controlWrapper");
            }
            writer.Write(">");

            RenderTextBoxSpan(writer);
        }

        private void RenderTextBoxSpan(HtmlTextWriter writer)
        {
            writer.WriteBeginTag("div");
            writer.Write(">");
            writer.WriteFullBeginTag("table cellpadding='1' cellspacing='0' border='0'");
            writer.WriteFullBeginTag("tr");
            writer.WriteFullBeginTag("td");
            writer.WriteBeginTag("span");
            if (!string.IsNullOrEmpty(CssClass))
            {
                writer.WriteAttribute("class", "textBoxWrapper " + CssClass);
            }
            else
            {
                writer.WriteAttribute("class", "textBoxWrapper");
            }

            writer.Write(">");

            writer.WriteFullBeginTag("table cellpadding='1' cellspacing='0' border='0'");
            writer.WriteFullBeginTag("tr");
            writer.WriteFullBeginTag("td");
            // text box
            writer.WriteBeginTag("input");
            writer.WriteAttribute("ID", this.ClientID + "_ddText");
            writer.WriteAttribute("Name", this.UniqueID + "$ddText");
            writer.WriteAttribute("type", "text");
            writer.WriteAttribute("readonly", null);

            if (!string.IsNullOrEmpty(CssClass))
            {
                writer.WriteAttribute("class", "ddTextBox " + CssClass);
            }
            else
            {
                writer.WriteAttribute("class", "ddTextBox");
            }
            writer.WriteAttribute("style", "width:" + Width.ToString());

            // set the value in the textbox if we have any
            ListItem item = this.Items.FindByValue(SelectedValue);
            if (item != null)
            {
                writer.WriteAttribute("value", item.Text);
                writer.WriteAttribute("title", item.Text);
            }

            writer.Write(" />");
            writer.WriteEndTag("td");
            writer.WriteFullBeginTag("td");
            // open dropdown arrow
            writer.WriteBeginTag("img");
            writer.WriteAttribute("src", ResolveClientUrl(ArrowImagePath));
            writer.WriteAttribute("align", "absmiddle");
            writer.WriteAttribute("ID", this.ClientID + "_btnOpenDropDown");
            writer.WriteAttribute("class", "arrowImg");
            writer.Write(" />");
            writer.WriteEndTag("td");
            writer.WriteEndTag("tr");
            writer.WriteEndTag("table");
            writer.WriteEndTag("span");
            // clearing div so the listbox does not show in the wrong place
            writer.WriteBeginTag("div");
            writer.WriteAttribute("style", "clear:both;");
            writer.Write(">");
            writer.WriteEndTag("div");

            // ending textbox wrapper div
            writer.WriteEndTag("div");

            // hidden value field
            writer.WriteBeginTag("input");
            writer.WriteAttribute("Id", this.ClientID + "_ddValue");
            writer.WriteAttribute("Name", this.UniqueID);
            writer.WriteAttribute("type", "text");
            writer.WriteAttribute("style", "display:none;");
            if (item != null)
                writer.WriteAttribute("value", item.Value);
            writer.Write(" />");

            // listbox wrapper div
            writer.WriteLine();

            writer.WriteBeginTag("div");
            writer.WriteAttribute("ID", this.ClientID + "_listBox");
            writer.WriteAttribute("class", "listBox");
            writer.WriteAttribute("tabindex", "0");
            writer.Write(">");
            writer.WriteLine();

            // opening ul... the rest is rendered in the RenderContents()
            writer.WriteFullBeginTag("ul");
        }


        protected override void RenderContents(HtmlTextWriter writer)
        {
            ListItemCollection items = this.Items;

            for (int j = 0; j < items.Count; j++)
            {
                ListItem item = items[j];

                writer.WriteBeginTag("li");
                writer.WriteAttribute("itemVal", item.Value, true);
                writer.WriteAttribute("itemIndex", j.ToString(), true);

                if (item.Selected)
                {
                    writer.WriteAttribute("selected", "selected");
                }

                if (item.Attributes != null && item.Attributes.Count > 0)
                {
                    item.Attributes.Render(writer);
                }

                if (this.Page != null)
                {
                    this.Page.ClientScript.RegisterForEventValidation(this.UniqueID, item.Value);
                }

                writer.Write(">");
                HttpUtility.HtmlEncode(item.Text, writer);
                writer.WriteEndTag("li");
                writer.WriteLine();
            }
        }


        public override void RenderEndTag(HtmlTextWriter writer)
        {
            // end of the ul of items
            writer.WriteEndTag("ul");
            writer.WriteEndTag("td");
            writer.WriteEndTag("tr");
            writer.WriteEndTag("table");

            // end of the listbox div
            writer.WriteEndTag("div");

            // end of the control div
            writer.WriteEndTag("div");
        }


        #endregion


        protected override void PerformDataBinding(IEnumerable dataSource)
        {
            string selectedValue = SelectedValue;
            base.PerformDataBinding(dataSource);
            SelectedValue = selectedValue;
        }

        #region IPostBackDataHandler Members


        public new bool LoadPostData(string postDataKey, NameValueCollection postCollection)
        {
            // get the new selected value from the postback data collection
            // and if it's different from the current one, select it onto our property
            // and return true, so we get a RisePostDataChangedEvent call
            string newSelectedValue = postCollection[postDataKey];
            if (newSelectedValue != null)
            {
                if (newSelectedValue != SelectedValue)
                {
                    SelectedValue = newSelectedValue;
                    return true;
                }
            }

            return false;
        }


        public new void RaisePostDataChangedEvent()
        {
            this.OnSelectedIndexChanged(EventArgs.Empty);
        }

        #endregion

        /// <summary>
        /// Gets or sets the arrow image path.
        /// </summary>
        /// <value>The arrow image path.</value>
        public string ArrowImagePath
        {
            get { return (string)ViewState["ArrowImagePath"] ?? "x"; }
            set { ViewState["ArrowImagePath"] = value; }
        }
    }
}
